// ----------------------------------------------------------------------------
//
//  Copyright (C) 2007-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#ifndef __SVHIPF_H
#define __SVHIPF_H


// 24 dB/oct highpass filer, using two state-variable
// filter sections.


// Filter configuration shared by all channels.
//
class Svhip_conf
{
public:

    Svhip_conf (void) : _f0 (0), _f1 (0), _g (1), _c1 (0), _c2 (0), _c3 (0), _c4 (0) {};
    ~Svhip_conf (void) {}

    // Set target -3dB frequency.
    // Calling process () changes the actual frequency in
    // small steps to avoid any clicks or other artefacts.
    void setfreq (float f) { _f1 = f; }
    void process (void);

private:

    friend class Svhip_filt;

    float _f0;                 // Current frequency.
    float _f1;                 // Target frequency.
    float _g;                  // Gain factor.
    float _c1, _c2, _c3, _c4;  // Filter coefficients.
};


// Single channel filter.
//
class Svhip_filt
{
public:

    Svhip_filt (void) { reset (); }
    ~Svhip_filt (void) {}

    // Reset interal state.
    void reset (void) { _z1 = _z2 = _z3 = _z4 = 0; }
    // Process n samples.
    void process (const Svhip_conf *C, int n, float *ip, float *op);

private:

    float _z1, _z2, _z3, _z4;  // Filter state. 
};


#endif
