// ----------------------------------------------------------------------------
//
//  Copyright (C) 2007-2017 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#ifndef __SVPARF_H
#define __SVPARF_H


// Parameteric filter section using a single state-variable section.
// Since in octoproc the filter configuration is fixed there is
// no smoothing of parameter changes in this implementation.


// Filter configuration shared by all channels.
// 
class Svpar_conf
{
public:

    Svpar_conf (void) : _a0 (0), _a1 (0), _b1 (0), _b2 (0) {};
    ~Svpar_conf (void) {}

    // Set center frequency, gain and bandwidth.
    // f = center frequency / sample rate.
    // g = linear (not dB) gain.
    // b = relative bandwidth.
    void setpars (float f, float g, float b);

private:

    friend class Svpar_filt;

    float  _a0, _a1, _b1, _b2;
    
};


// Single channel filter.
//
class Svpar_filt
{
public:

    Svpar_filt (void) { reset (); }
    ~Svpar_filt (void) {}

    // Reset internal state.
    void reset (void) { _z1 = _z2 = 0; }
    // Process n samples.
    void process (const Svpar_conf *C, int n, float *ip, float *op);

private:

    float _z1, _z2; // Filter state 
};


#endif
